@@ -1,9 +1,11 @@
-/* ===========================================================================
- * bas55, an implementation of the Minimal BASIC programming language.
- *
- * Virtual machine that can execute the BASIC program compiled in code.c, str.c
+/* --------------------------------------------------------------------------
+ * Copyright (C) 2023 Jorge Giner Cordero
+ * License: GNU GPL version 3 or later <https://gnu.org/licenses/gpl.html>
+ * --------------------------------------------------------------------------
+ */
+
+/* Virtual machine that can execute the BASIC program compiled in code.c, str.c
  * and data.c .
- * ===========================================================================
  */
 
 #include <config.h>
@@ -456,6 +458,14 @@ static void let_strvar_op(void)
 		set_rampos_inited(rampos);
 	}
 	stri = s_stack[--s_sp].i;
+
+	if (strlen(strings[stri]->str) > STR_VAR_MAX_CHARS) {
+		eprintln(E_STR_DATUM_TOO_LONG, s_cur_line_num);
+		enl();
+		s_fatal = 1;
+		return;
+	}
+
 	oldi = s_ram[rampos].i;
 	if (oldi != stri) {
 		dec_string_refcount(oldi);
@@ -719,6 +729,7 @@ static void read_strvar_op(void)
 	if (s_debug_mode) {
 		set_rampos_inited(rampos);
 	}
+
 	if ((ecode = read_data_str(&stri, NULL)) != 0) {
 		eprintln(ecode, s_cur_line_num);
 		enl();
@@ -726,6 +737,13 @@ static void read_strvar_op(void)
 		return;
 	}
 
+	if (strlen(strings[stri]->str) > STR_VAR_MAX_CHARS) {
+		eprintln(E_STR_DATUM_TOO_LONG, s_cur_line_num);
+		enl();
+		s_fatal = 1;
+		return;
+	}
+
 	oldi = s_ram[rampos].i;
 	if (stri != oldi) {
 		dec_string_refcount(oldi);
@@ -1166,11 +1184,12 @@ static int s_input_pc;
 static int s_input_pass;
 static int s_input_comma;
 static const char *s_input_p;
-static char s_input_line[LINE_MAX_CHARS + 3];
+static char s_input_line[LINE_MAX_CHARS + 1];
 
 static void input_op(void)
 {
 	int r;
+	size_t eindex;
 
 	s_input_pass = 1;
 	s_input_pc = s_pc - 1;
@@ -1187,6 +1206,16 @@ retry:	r = get_line("? ", s_input_line, sizeof s_input_line, stdin);
 		enl();
 		goto retry;
 	}
+       
+	toupper_str(s_input_line);
+	if (chk_basic_chars(s_input_line, strlen(s_input_line), 1, &eindex)) {
+		eprint(E_INVAL_CHARS);
+		putc('(', stderr);
+		putc(s_input_line[eindex], stderr);
+		putc(')', stderr);
+		enl();
+		goto retry;
+	}
 
 	s_input_p = s_input_line;
 }
@@ -1281,6 +1310,8 @@ static void input_str_op_pass1(void)
 		if (t == DATA_ELEM_QUOTED_STR &&
 		    delem.str.start[delem.str.len] != '\"') {
 			retry_input(E_STR_NOEND);
+		} else if (delem.str.len > STR_VAR_MAX_CHARS) {
+			retry_input(E_STR_DATUM_TOO_LONG);
 		} else {
 			/* Consume and check next token, that must be a comma
 			 * or EOF. */
